/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_RTCIceCandidateBinding_h__
#define mozilla_dom_RTCIceCandidateBinding_h__

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/BindingUtils.h"
#include "mozilla/dom/CallbackInterface.h"
#include "mozilla/dom/Nullable.h"
#include "nsWeakReference.h"

namespace mozilla {
namespace dom {

class NativePropertyHooks;
class ProtoAndIfaceArray;
struct RTCIceCandidateInit;
struct RTCIceCandidateInitAtoms;
class mozRTCIceCandidate;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct RTCIceCandidateInit : public MainThreadDictionaryBase
{
  nsString mCandidate;
  Optional<uint16_t > mSdpMLineIndex;
  nsString mSdpMid;

  RTCIceCandidateInit();

  explicit inline RTCIceCandidateInit(const RTCIceCandidateInit& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline RTCIceCandidateInit(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, RTCIceCandidateInitAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const RTCIceCandidateInit& aOther);
};

namespace dictionary_detail {
struct FastRTCIceCandidateInit : public RTCIceCandidateInit
{
  inline FastRTCIceCandidateInit()
    : RTCIceCandidateInit(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace dictionary_detail


namespace mozRTCIceCandidateBinding {

  typedef mozilla::dom::mozRTCIceCandidate NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];
  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  bool
  ConstructorEnabled(JSContext* /* unused */, JS::Handle<JSObject*> /* unused */);

  const JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, mozilla::dom::mozRTCIceCandidate* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, T* aObject)
  {
    return Wrap(aCx, aScope, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, ProtoAndIfaceArray& aProtoAndIfaceArray, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace mozRTCIceCandidateBinding



class mozRTCIceCandidateJSImpl : public CallbackInterface
{
public:
  explicit inline mozRTCIceCandidateJSImpl(JS::Handle<JSObject*> aCallback, nsIGlobalObject* aIncumbentGlobal)
    : CallbackInterface(aCallback, aIncumbentGlobal)
  {
  }

  void __Init(const RTCIceCandidateInit& candidateInitDict, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  inline bool
  operator==(const mozRTCIceCandidateJSImpl& aOther) const
  {
    return CallbackInterface::operator==(aOther);
  }

  void GetCandidate(nsString& aRetVal, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void GetSdpMid(nsString& aRetVal, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  Nullable<uint16_t > GetSdpMLineIndex(ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void SetCandidate(const nsAString& arg, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void SetSdpMid(const nsAString& arg, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void SetSdpMLineIndex(const Nullable<uint16_t >& arg, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);
};


class mozRTCIceCandidate MOZ_FINAL : public nsSupportsWeakReference,
                                     public nsWrapperCache
{
public:
  NS_DECL_CYCLE_COLLECTING_ISUPPORTS
  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(mozRTCIceCandidate)

private:
  nsRefPtr<mozRTCIceCandidateJSImpl> mImpl;
  nsCOMPtr<nsISupports> mParent;

public:
  mozRTCIceCandidate(JS::Handle<JSObject*> aJSImplObject, nsPIDOMWindow* aParent);

  nsISupports* GetParentObject() const;

  virtual JSObject* WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;

  static already_AddRefed<mozRTCIceCandidate> Constructor(const GlobalObject& global, JSContext* cx, const RTCIceCandidateInit& candidateInitDict, ErrorResult& aRv);

  void GetCandidate(nsString& aRetVal, ErrorResult& aRv, JSCompartment* aCompartment = nullptr) const;

  void SetCandidate(const nsAString& arg, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  void GetSdpMid(nsString& aRetVal, ErrorResult& aRv, JSCompartment* aCompartment = nullptr) const;

  void SetSdpMid(const nsAString& arg, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  Nullable<uint16_t > GetSdpMLineIndex(ErrorResult& aRv, JSCompartment* aCompartment = nullptr) const;

  void SetSdpMLineIndex(const Nullable<uint16_t >& arg, ErrorResult& aRv, JSCompartment* aCompartment = nullptr);

  static bool
  _Create(JSContext* cx, unsigned argc, JS::Value* vp);
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_RTCIceCandidateBinding_h__
